require 'fileutils'

desc "ALL, CONTROLLER, VIEW, MODEL, UI are valid options."
task 'generate'
rule(/^generate/) do |t|
  ARGV[1..-1].each do |generator_command|
    command, argument = generator_command.split("=")
    case command
    when "ALL"
      generate_tuple argument
    when "VIEW"
      generate_view argument
    when "CONTROLLER"
      generate_controller argument
    when "MODEL"
      generate_model argument 
    when "UI"
      generate_ui argument 
      
    else
      $stdout << "Unknown generate target #{argument}"
    end
  end
end

def generate_tuple path
  pwd = FileUtils.pwd
  generate_controller path
  FileUtils.cd pwd
  generate_model path
  FileUtils.cd pwd
  generate_view path, using_ui = true
  FileUtils.cd pwd
  generate_ui path  
end

def generate_controller path
  name = setup_directory path
  file_name = "#{name}_controller.rb"
  name = camelize name 
  $stdout << "Generating controller #{name}Controller in file #{file_name}\n"
  File.open(file_name, "w") do |controller_file|
  controller_file << <<-ENDL
class #{name}Controller < ApplicationController
  set_model '#{name}Model'
  set_view '#{name}View'
  set_close_action :exit
end
  ENDL
  end
end

def generate_model(path)
  name = setup_directory(path)
  file_name = "#{name}_model.rb"
  name = camelize(name)
  $stdout << "Generating model #{name}Model in file #{file_name}\n"
  File.open(file_name, "w") do |model_file|
  model_file << <<-ENDL
class #{name}Model

end
  ENDL
  end
end

def generate_view path, using_ui = false
  name = setup_directory path
  file_name = "#{name}_view.rb"
  cname = camelize(name)
  puts "Generating view #{cname}View in file #{file_name}"
  
  
  require_ui = using_ui ? "require '#{name}_ui'" : ''
  java_class = using_ui ? "#{cname}Frame" : "'REPLACE_ME' "
  view_file = %~
#{require_ui}

class #{cname}View < ApplicationView
  set_java_class  #{java_class}

  # Put your load behavior here:
  def load
     move_to_center
  end

end
  ~
  
  File.open(file_name, "w") {|f| f.puts view_file}
  
  
end



def generate_ui path 
  name = setup_directory path 
  file_name = "#{name}_ui.rb"
  name = camelize(name)
  $stdout << "Generating ui #{name}Frame in file #{file_name}\n"
  source = %~# Must have the Neurogami::SwingSet files in your project
require 'swingset'  

include  Neurogami::SwingSet::Core

class #{name}Frame < Frame   
  FRAME_WIDTH  = 420
  FRAME_HEIGHT = 430

  LABEL_HEIGHT = 20
  LABEL_WIDTH  = 200

  # Need accessor methods for you UI components:  
  attr_accessor :example_label, :example_value


  def initialize *args
    super
    self.minimum_width  = FRAME_WIDTH
    self.minimum_height = FRAME_HEIGHT
    self.setSize FRAME_HEIGHT, FRAME_WIDTH 
    set_up_components
  end


  def set_up_components
    # Assumes you have the MiG layout jar in your project and
    # bundled with the app.
    
    component_panel = Panel.new

    component_panel.background_color 255, 255, 255
    component_panel.size FRAME_WIDTH, FRAME_HEIGHT

    @example_label = Label.new do |l|
      l.font = Font.new "Lucida Grande", 0, 18
      l.minimum_dimensions LABEL_WIDTH, LABEL_HEIGHT
      l.text = "Example label"
    end

    @example_value = Label.new do |l|
      l.font = Font.new "Lucida Grande", 0, 18
      l.minimum_dimensions LABEL_WIDTH, LABEL_HEIGHT
      l.text = "An example"
    end

    component_panel.add @example_label, 'grow x'
    component_panel.add @example_value, "gap unrelated"

    add component_panel

  end
end ~

  File.open(file_name, "w"){ |f| f.puts source } 
end

def setup_directory(path)
  FileUtils.mkdir_p path.gsub("\\", "/")
  FileUtils.cd(path)
  path.split("/").last
end

def camelize(name, first_letter_in_uppercase = true)
  name = name.to_s
  if first_letter_in_uppercase
    name.gsub(/\/(.?)/) { "::" + $1.upcase }.gsub(/(^|_)(.)/) { $2.upcase }
  else
    name[0..0] + camelize(name[1..-1])
  end
end

