require 'jimpanzee/view_nesting'

describe Jimpanzee::Nesting do
  it 'creates PropertyNesting with :sub_view and :view' do
    Jimpanzee::Nesting.new(:view => "foo", :sub_view => "bar").should be_an_instance_of(Jimpanzee::PropertyNesting)
  end
  it 'creates MethodNesting with :sub_view and :using' do
    Jimpanzee::Nesting.new(:sub_view => "foo", :using => ["add_foo", "remove_foo"]).should be_an_instance_of(Jimpanzee::MethodNesting)
  end
end

describe Jimpanzee::MethodNesting do
  it 'calls the first/only :using method on add' do
    nesting = Jimpanzee::Nesting.new(:sub_view => "foo", :using => [:add_foo, :remove_foo])
    view = mock("view")
    
    view.should_receive :add_foo
    
    nesting.add(view, nil, nil, nil, nil)
  end
  
  it 'calls the second :using method on remove' do
    nesting = Jimpanzee::Nesting.new(:sub_view => "foo", :using => [:add_foo, :remove_foo])
    view = mock("view")
    
    view.should_receive :remove_foo
    
    nesting.remove(view, nil, nil, nil, nil)
  end
end

describe Jimpanzee::PropertyNesting do
  it 'calls add on the :view container component on add' do
    nesting = Jimpanzee::Nesting.new(:view => "foo", :sub_view => "bar")
    view = mock("view")
    view_component = mock("view_component")
    view.stub!(:foo).and_return view_component
    
    nested_component = mock("nested_component")
    view_component.should_receive(:add).with(nested_component)
    
    nesting.add(view, nil, nested_component, nil, nil)
  end
  
  it 'calls remove on the :view container component on remove' do
    nesting = Jimpanzee::Nesting.new(:view => "foo", :sub_view => "bar")
    view = mock("view")
    view_component = mock("view_component")
    view.stub!(:foo).and_return view_component
    
    nested_component = mock("nested_component")
    view_component.should_receive(:remove).with(nested_component)
    
    nesting.remove(view, nil, nested_component, nil, nil)
  end
end